import argparse
import os
import math
import re
from datetime import date, datetime, timedelta
import subprocess

# args - d : days of logs and indexes - int

def get_data_size(required_list):
    total_size = 0
    for file in required_list:
        if os.path.isdir(file):
            output_stream = os.popen('du -s ' + file)
            output = output_stream.read().split('\t')[0]   # Size in KB
            output_stream.close()
            total_size += int(output)
        else:
            total_size += float(os.path.getsize(file)) / 1024 # Convect Byte to KB

    return total_size

# FetchedFiles is in INDEXERDIR in CMA/MGMT/MDS
def handle_fetched_files(fwdir_path = os.getenv('FWDIR')):
    version = re.search('\d+', fwdir_path).group(0)
    
    if int(version) >= 81:
        if os.path.exists(os.getenv('INDEXERDIR') + '/data/FetchedFiles'):
            return get_data_size([os.getenv('INDEXERDIR') + '/data/FetchedFiles'])
    return None            

def get_list_from_path(path, date_regex, date_format):
    if(os.path.exists(path) == False):
        return []
    
    earliest_date_we_need = None

    if args.d is not None:
        earliest_date_we_need = date.today() - timedelta(days=args.d)

    required_files = []

    for file in os.listdir(path):
        # Remove elg, csv, json files
        if re.search('\.elg', file) is not None or file.endswith(('csv', 'json')):
            continue

        if args.d is None:
            required_files.append(path + '/' + file)
            continue

        date_location = re.search(date_regex, file)
        # If we have date regex in the string
        if date_location is None:
            required_files.append(path + '/' + file)
            continue

        # Handle in case of 2023-77-52 problem
        try:
            file_date = datetime.strptime(date_location.group(), date_format).date()
            # If date is range
            if file_date > earliest_date_we_need:
                required_files.append(path + '/' + file)
        except ValueError:
            required_files.append(path + '/' + file)
    
    return required_files

def get_cma_size(fwdir_path = os.getenv('FWDIR')):
    cma_size = 0

    # FetchedFiles
    fetched_files_size = handle_fetched_files()
    if fetched_files_size is not None:
        cma_size += fetched_files_size

    # Logs
    logs_path = fwdir_path + '/log'
    logs_date_regex  = '\d{4}-\d{2}-\d{2}'
    logs_date_format = '%Y-%m-%d'
    required_logs_list = get_list_from_path(logs_path, logs_date_regex, logs_date_format)

    # Remove blob folder to  calculate later
    if logs_path + '/blob' in required_logs_list:
        required_logs_list.remove(logs_path + '/blob')

    cma_size += get_data_size(required_logs_list)
    
    blobs_path = logs_path + '/blob'
    blobs_date_regex  = '\d{8}'
    blobs_date_format = '%Y%m%d'
    required_blobs_list = get_list_from_path(blobs_path, blobs_date_regex, blobs_date_format)

    cma_size += get_data_size(required_blobs_list)

    # Indexes
    indexes_path = os.getenv('RTDIR') + '/log_indexes'
    indexes_date_regex  = logs_date_regex
    indexes_date_format = logs_date_format
    required_indexes_list = get_list_from_path(indexes_path, indexes_date_regex, indexes_date_format)

    # Remove template folder
    if indexes_path + '/template' in required_indexes_list:
        required_indexes_list.remove(indexes_path + '/template')

    cma_size += get_data_size(required_indexes_list)

    return cma_size

def main():    
    # Setup args
    global args
    parser = argparse.ArgumentParser(description='Calculate Logs & indexes size')
    parser.add_argument('-d', type=int, help='Number of days for logs and indexes')
    args = parser.parse_args()

    if args.d and args.d < 0:
        raise argparse.ArgumentTypeError("-d should be a positive number")

    # Calculate the size of the MDS/MGMT
    main_size = get_cma_size()
    data_size = main_size
    
    # In case of MDS, calculate the size of all the CMA's
    if os.getenv('MDSDIR') is not None:
        cma_list = os.listdir(os.getenv('MDSDIR')+'/customers/')

        for cma in cma_list:
            command = "source {}/tmp/.CPprofile.sh &> /dev/null; mdsenv {};".format(os.getenv('MDS_CPDIR'), cma)
            command += " printenv FWDIR"
            p = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            fwdir_path = p.communicate()[0].decode('UTF-8')[:-1]
            cur_size = get_cma_size(fwdir_path)
            data_size+=cur_size

    data_size_in_gb = float(data_size) / (1024 * 1024)
    print(int(math.ceil(float(data_size_in_gb))))
    
if __name__ == "__main__":
    main()