import re
import subprocess
import datetime
import xml.etree.ElementTree as ET
import os
from threading import Timer

format_definition = {"syslog": "SyslogFormatDefinition.xml", "cef": "CefFormatDefinition.xml",
                     "leef": "LeefFormatDefinition.xml", "generic": "GenericFormatDefinition.xml",
                     "splunk": "SplunkFormatDefinition.xml", "logrhythm": "LogRhythmFormatDefinition.xml",
                     "json": "JsonFormatDefinition.xml", "rsa": "RsaFormatDefinition.xml",
                     "laas": "JsonFormatDefinition.xml"}
fields_mapping = {"cef": "CefFieldsMapping.xml", "leef": "LeefFieldsMapping.xml", "generic": "GenericFieldsMapping.xml",
                  "splunk": "SplunkFieldsMapping.xml", "logrhythm": "LogRhythmFieldsMapping.xml",
                  "json": "JsonFieldsMapping.xml", "rsa": "RsaFieldsMapping.xml", "laas": "LaaSFieldsMapping.xml"}

log_file = "log_export_transition.log"
empty_string = ""
regex_to_catch_version_string = "R\\d*.\\d*"


def is_mds():
    return os.getenv('MDSDIR') is not None

# This function will take an old version path (IE /opt/CPmds-R81.20/...) and will replace the version part (RXX.XX) to the
# new version currently on the server
def check_cert_in_new_path(path):
    new_path = empty_string
    new_ver = re.search(regex_to_catch_version_string, os.getenv("CPDIR")).group()
    # check if the current path has any 'version-related' strings IE. CPxxxx-R80.40
    if re.search("CP.*-" + regex_to_catch_version_string, path) is not None:
        # if so, we replace the version string with the new current version
        new_path = str(re.sub(regex_to_catch_version_string, new_ver, path))
    # then we return it as the output, or just return a string
    return new_path


def xml_to_string(file):
    root = ET.parse(file).getroot()
    str = ET.tostring(root)
    return re.sub('\\n|\\t|\\r', '', str.decode())


def compare_XMLs(file_name1, file_name2):
    debug_message("filename1 : {}, filename2:{}".format(file_name1, file_name2))
    try:
        return xml_to_string(file_name2) == xml_to_string(file_name1)
    except Exception:
        with open(file_name1) as file1:
            file1_txt = file1.read()
        with open(file_name2) as file2:
            file2_txt = file2.read()
        file1_txt = re.sub('\\n|\\t|\\r', '', file1_txt)
        file2_txt = re.sub('\\n|\\t|\\r', '', file2_txt)
        return file1_txt == file2_txt


def debug_message(msg):
    # print(msg)
    with open(log_file, 'a') as logger:
        try:
            logger.write(str(datetime.datetime.now()) + " : " + str(msg) + "\n")
        except TypeError:
            logger.write("a print had a non string in it, failed to convert")
        except:
            logger.write("print raised a non type error")




def list_dir(path):
    if not os.path.isdir(path):
        debug_message("{} is not a directory".format(path))
        return []
    return os.listdir(path)


def symlink_bin(src_dir, dst_dir, proc_name):
    # if the bin file in the destination is a symlink, exit.
    if is_symlink(dst_dir + proc_name):
        return False

    if os.path.isfile(dst_dir + proc_name):
        os.remove(dst_dir + proc_name)

    os.symlink(src_dir + proc_name, dst_dir + proc_name)
    return True


def update_cert(target_dir):
    """update certificate path on targetConfiguration.xml"""
    target_conf_file = "{}targetConfiguration.xml".format(target_dir)
    debug_message("target_conf_file: {}".format(target_conf_file))
    try:
        tree = ET.parse(target_conf_file)
        root = tree.getroot()

        transport = root.find("destination").find("transport")
        public_cloud_ca_file = transport.find("public_cloud_ca_file")
        pem_ca_file = transport.find("pem_ca_file")
        with open(target_conf_file, 'r') as file:
            filedata = file.read()

        if public_cloud_ca_file != None and public_cloud_ca_file.text:
            debug_message("public_cloud_ca_file found on {}".format(target_conf_file))
            public_ca = "{}{}".format(os.getenv("CPDIR"), "/conf/ca-bundle-public-cloud.crt")
            filedata = re.sub(r'<public_cloud_ca_file>.*</public_cloud_ca_file>',
                              "<public_cloud_ca_file>{}</public_cloud_ca_file>".format(public_ca), filedata)
        # This 'if' will check if the current path of the PEM file within the targetConfiguration.XML is legit (file exists)
        # If so, it will remain as is, if it is not there, we check if the file is in a
        # new version path (IE /opt/CPmds-R80.20/... -> /opt/CPmds-R81.20/...), if so we change the version in the path to the current version,
        # If it is not there as well, we go to the default ca-bundle.crt file with in the $CPDIR of the server/ relevant domain
        if pem_ca_file != None and pem_ca_file.text:
            if not os.path.isfile(pem_ca_file.text):
                new_file_to_check = check_cert_in_new_path(pem_ca_file.text)
                # debug_message("Checking if the file exist on different version folder...")
                if os.path.isfile(new_file_to_check):
                    # debug_message("Replacing")
                    filedata = re.sub(r'<pem_ca_file>.*</pem_ca_file>',
                              "<pem_ca_file>{}</pem_ca_file>".format(new_file_to_check), filedata)
                else:
                    debug_message("pem_ca_file found on {}".format(target_conf_file) + " but the file itself does not exists")
                    bundle_ca = "{}{}".format(os.getenv("CPDIR"), "/conf/ca-bundle.crt")
                    filedata = re.sub(r'<pem_ca_file>.*</pem_ca_file>',
                              "<pem_ca_file>{}</pem_ca_file>".format(bundle_ca), filedata)

        with open(target_conf_file, 'w') as file:
            file.write(filedata)

    except Exception as e:
        return


def target_transition(main_log_exp_dir, target_log_exp_dir, default_format_files):
    debug_message("target_transition main:{}, target_dir:{}\n".format(main_log_exp_dir, target_log_exp_dir))
    target_conf_dir = "{}{}/".format(target_log_exp_dir, "conf")

    update_cert(target_log_exp_dir)
    if not symlink_bin(main_log_exp_dir, target_log_exp_dir, "/log_exporter"):
        return False

    for file in list_dir(target_conf_dir):
        debug_message("target_transition file:{}\n".format(file))
        if file in default_format_files and (file.endswith("Mapping.xml") or file.endswith("Definition.xml")):
            format_file = target_conf_dir + file
            debug_message("format file:{}".format(format_file))
            if compare_XMLs(format_file, "{}/{}/{}".format(main_log_exp_dir, "conf", file)):
                debug_message("XMLs are the same ,removing XML from target conf")
                os.remove(format_file)
    return True


def targets_transition(main_log_exp_dir, targets_log_exp_dir, default_format_files):
    debug_message("targets_transition main:{}, targets:{}\n".format(main_log_exp_dir, targets_log_exp_dir))
    targets = list_dir(targets_log_exp_dir)
    for target in targets:
        debug_message("target: {}\n".format(target))
        target_dir = "{}{}/".format(targets_log_exp_dir, target)

        if not target_transition(main_log_exp_dir, target_dir, default_format_files):
            continue
        update_tagretConfiguration(target_dir)


def update_tagretConfiguration(targrt_dir):
    """ If the customer using the format file at target/conf/, targetConfiguration.xml will be updated accordingly"""

    target_conf_file = "{}/targetConfiguration.xml".format(targrt_dir)

    if not os.path.exists(target_conf_file):
        return False

    tree = ET.parse(target_conf_file)
    root = tree.getroot()

    format = root.find("format")
    dest_type = format.get("type").lower()
    mapping_conf = format.find("resolver").find("mappingConfiguration")
    format_fields = format.find("formatHeaderFile")

    with open(target_conf_file, 'r') as file:
        filedata = file.read()

    if not mapping_conf.text:
        customized_mapping_file = "conf/{}".format(fields_mapping.get(dest_type))
        debug_message("customized_mapping_file:: " + customized_mapping_file)
        # if mapping file exist in target/conf
        if os.path.isfile("{}/{}".format(targrt_dir, customized_mapping_file)):
            filedata = re.sub(r'<mappingConfiguration>.*</mappingConfiguration>',
                              "<mappingConfiguration>{}</mappingConfiguration>".format(customized_mapping_file),
                              filedata)
            debug_message("mapping")
    if not format_fields.text:
        customized_mapping_file = "conf/{}".format(format_definition.get(dest_type))
        debug_message("customized_mapping_file:: " + customized_mapping_file)
        # if foramt file exist in target/conf
        if os.path.isfile("{}/{}".format(targrt_dir, customized_mapping_file)):
            filedata = re.sub(r'<formatHeaderFile>.*</formatHeaderFile>',
                              "<formatHeaderFile>{}</formatHeaderFile>".format(customized_mapping_file), filedata)
            debug_message("format ")

    with open(target_conf_file, 'w') as file:
        file.write(filedata)


def is_symlink(file):
    return os.path.islink(file)


def getAllDomains():
    root_dir = os.getenv('MDSDIR') + "/customers/"
    domains_list = []
    for domain_candidate in list_dir(root_dir):
        domain_candidate_full_path = "{}/{}".format(root_dir, domain_candidate)
        if os.path.isdir(domain_candidate_full_path):
            domains_list.append(domain_candidate)
    return domains_list


def run_command(command):
    debug_message("Running the command: {}".format(command))
    p = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    output = p.communicate()[0]
    return output.decode().strip()


def get_proc_dir(domain_name, proc_name):
    cpdir = os.getenv('MDS_CPDIR')
    return run_command(
        "source {}/tmp/.CPprofile.sh &> /dev/null; mdsenv {}; cd ${}; pwd;".format(cpdir, domain_name, proc_name))


def mds_transition(main_log_exp_dir, default_format_files):
    domains = getAllDomains()
    main_log_indexer = os.getenv('RTDIR') + "/log_indexer"
    for domain in domains:
        # debug_message("working on domain " + domain)
        log_exporter = get_proc_dir(domain, "EXPORTERDIR")
        log_indexer = get_proc_dir(domain, "INDEXERDIR")
        debug_message(log_exporter)
        symlink_bin(main_log_indexer, log_indexer, "/log_indexer")
        symlink_bin(main_log_exp_dir, log_exporter, "/log_exporter")

        targets_log_exp_dir = get_proc_dir(domain, 'EXPORTERDIR')
        targets_transition(main_log_exp_dir, "{}/{}/".format(targets_log_exp_dir, "targets"), default_format_files)


def main():
    debug_message("Started to upgrade all log exporters")
    main_log_exp_dir = os.getenv('RTDIR') + "/log_exporter"
    default_format_files = list_dir("{}/{}/".format(main_log_exp_dir, "conf"))
    debug_message(default_format_files)
    debug_message("EXPORTERDIR: {}".format(main_log_exp_dir))
    if is_mds():
        debug_message("isMDS")
        mds_transition(main_log_exp_dir, default_format_files)

    targets_transition(main_log_exp_dir, "{}/{}/".format(main_log_exp_dir, "targets"), default_format_files)
    debug_message("Finished the run")


if __name__ == '__main__':
    main()